/*******************************************************************************
* Licensed Materials - Property of HCL
* (c) Copyright HCL Technologies Ltd. 2019.  All Rights Reserved.
*
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp.
*******************************************************************************/
/**
 * Sample configuration file for running LIVE with default settings using PM2
 */

module.exports = {
  /**
   * Application configuration section
   * http://pm2.keymetrics.io/docs/usage/application-declaration/
   */
  apps : [
    
    {
      name: 'LiveAuthServer',
      script: '../liveAuthServer/app.js',
      node_args: '--max_old_space_size=4096',
      error_file: "../logs/liveauth-err.log",
      out_file: "../logs/liveauth-out.log",
      log_date_format: "YYYY-MM-DD HH:mm Z",
	  merge_logs: true,
      env: { /* Default environment is for production mode */
          DR_LOG_NO_TIMESTAMPS: true,
          NODE_ENV: 'production'
      },
      env_development: { /* This environment is for development mode */
          DR_LOG_NO_TIMESTAMPS: true,
          NODE_ENV: 'development'
      }
    },
{
      name: 'LiveHomeServer',
      script: '../liveHomeServer/app.js',
      node_args: '--max_old_space_size=4096',
      error_file: "../logs/livehome-err.log",
      out_file: "../logs/livehome-out.log",
      log_date_format: "YYYY-MM-DD HH:mm Z",
      merge_logs: true,
	  
	  env: { /* Default environment is for production mode */
          DR_LOG_NO_TIMESTAMPS: true,
          NODE_ENV: 'production'
      },
      env_development: { /* This environment is for development mode */
          DR_LOG_NO_TIMESTAMPS: true,
          NODE_ENV: 'development'
      }
    },
{
      name: 'LiveAudioServer',
      script: 'server.js',
	  cwd: '../liveHomeServer/node_modules/rtcmulticonnection/',
      node_args: '--max_old_space_size=4096',
      error_file: "../../../logs/liveaudio-err.log",
      out_file: "../../../logs/liveaudio-out.log",
      log_date_format: "YYYY-MM-DD HH:mm Z",
      merge_logs: true,
	  
	  env: { /* Default environment is for production mode */
          DR_LOG_NO_TIMESTAMPS: true,
          NODE_ENV: 'production'
      },
      env_development: { /* This environment is for development mode */
          DR_LOG_NO_TIMESTAMPS: true,
          NODE_ENV: 'development'
      }
    }     
  ],

  /**
   * Deployment section
   * http://pm2.keymetrics.io/docs/usage/deployment/
   */
  deploy : {
    production : {
      user : 'node',
      host : '212.83.163.1',
      ref  : 'origin/master',
      repo : 'git@github.com:repo.git',
      path : '/var/www/production',
      'post-deploy' : 'npm install && pm2 reload ecosystem.config.js --env production'
    },
    dev : {
      user : 'node',
      host : '212.83.163.1',
      ref  : 'origin/master',
      repo : 'git@github.com:repo.git',
      path : '/var/www/development',
      'post-deploy' : 'npm install && pm2 reload ecosystem.config.js --env dev',
      env  : {
        NODE_ENV: 'dev'
      }
    }
  }
};
