/************************************************************************/
/* Licensed Materials – Property of  HCL                                */
/* (c) Copyright HCL Technologies Ltd. 2016, 2017 All rights reserved.  */
/* Trademark of HCL Technologies Limited                                */
/************************************************************************/

const fs = require('fs-extra');
const async = require('async');
var workManager = require('../modules/workManager');

module.exports = function () {
	var Exporter = {};
	//Exports all whiteboards
	Exporter.doExportAll = function (expImpPath) {
		var config = require('../modules/config');
		const fs = require('fs-extra');
		var Rooms = require('../models/rooms');

		async.waterfall([
			// find Room in DB
			function (callback) {
				workManager.workStarted();
				Rooms.find({}, function (err, room) {
					callback(err, room);
				});
				
			},
			function (room, callback) {
				fs.ensureDir(expImpPath, function (err) {
					callback(err, room);
				});				
			},
			function (room, callback) {
				var roomFile = expImpPath + '/' + 'rooms.json';
				winston.info('Exporting Rooms to - ', roomFile);
				fs.outputJson(roomFile, room[0], { spaces: 4 }, function (err) {
					callback(err, room);
				});
			},
			function (room, callback) {
				//winston.info(room[0].whiteboards);
				room[0].whiteboards.forEach(function (wb) {
					workManager.workStarted();
					var dbName = wb.graph_id;
					var DB = require('../models/db')(dbName);
					DB.find({}, function (err, db) {
						if (err) {
							DB.closeConnection();
							throw err;
						}
						
						workManager.workStarted();
						var dbFile = expImpPath + '/' + dbName + '.json';
						fs.outputJson(dbFile, db, { spaces: 4 }, function (err) {
							DB.closeConnection();
							if (err) throw err;
							winston.info('Exported Whiteboard', wb.wname, ' to - ', dbFile);
							workManager.workEnded();
						});

						workManager.workEnded();

					});

				});
				callback(null);
			},
			function (callback) {
				workManager.workStarted();
				var ModelElements = require('../models/modelElement');
				ModelElements.find({}, function (err, modelElements) {
					if (err) {
						ModelElements.closeConnection();
						throw err;
					}
					
					workManager.workStarted();
					var meFile = expImpPath + '/' + 'modelElements.json';
					fs.outputJson(meFile, modelElements, { spaces: 4 }, function (err) {
						ModelElements.closeConnection();
						if (err) throw err;
						winston.info('Exported Model Elements to - ', meFile);
						workManager.workEnded();
					});

					workManager.workEnded();

				});
				callback(null);
			}
		],
			function (err, result) {
				// all done
				if (Rooms != null) {
					Rooms.closeConnection();
				}

				workManager.workEnded();
		});

	};
	Exporter.doExportPartial = function (expImpPath, whiteboards) {
		var config = require('../modules/config');
		const fs = require('fs-extra');
		var Rooms = require('../models/rooms');

		async.waterfall([
			// find Room in DB
			function (callback) {
				workManager.workStarted();
				Rooms.find({}, function (err, room) {
					callback(err, room);
				});
				
			},
			function (room, callback) {
				fs.emptyDir(expImpPath, function(err){
					if (err)
						callback(err)
					fs.ensureDir(expImpPath, function (err) {
						callback(err, room);
					});					
				})				
			},
			function (room, callback) {
				var roomFile = expImpPath + '/' + 'rooms.json';
				var temp_room = room;
				winston.info('Exporting Rooms to - ', roomFile);
				var whiteboard_template_id =null;
				for(var i=0; i<temp_room[0].whiteboards.length;i++){
					if(temp_room[0].whiteboards[i].wname == 'Whiteboard Templates'||!whiteboards.includes(temp_room[0].whiteboards[i].graph_id)){
						winston.info("Deleting graph with name : "+temp_room[0].whiteboards[i].wname);
						//Should not happen but just in case(Command line check)
						if(temp_room[0].whiteboards[i].wname == 'Whiteboard Templates')
							whiteboard_template_id = room[0].whiteboards[i].graph_id;
						temp_room[0].whiteboards.splice(i,1);
						i--;
					}

				}//Should not happen but still(Command line check) 
				if(whiteboard_template_id && whiteboards.includes(whiteboard_template_id)){
					whiteboards.splice(whiteboards.indexOf(whiteboard_template_id),1);
				}
				fs.outputJson(roomFile, temp_room[0], { spaces: 4 }, function (err) {
					callback(err, whiteboards);
				});
			},
			function (wbs, callback) {
				//winston.info(room[0].whiteboards);
				wbs.forEach(function (wb) {
					workManager.workStarted();
					var dbName = (typeof whiteboards === 'undefined') ? wb.graph_id : wb;
					var DB = require('../models/db')(dbName);
					DB.find({}, function (err, db) {
						if (err) {
							DB.closeConnection();
							throw err;
						}
						
						workManager.workStarted();
						var dbFile = expImpPath + '/' + dbName + '.json';
						fs.outputJson(dbFile, db, { spaces: 4 }, function (err) {
							DB.closeConnection();
							if (err) throw err;
							winston.info('Exported Whiteboard', dbName, ' to - ', dbFile);
							workManager.workEnded();
						});

						workManager.workEnded();

					});

				});
				callback(null, wbs);
			},
			function (wbs,callback) {
				workManager.workStarted();
				var ModelElements = require('../models/modelElement');
				var ref_model_ids_to_retrieve=[];
				var rejected_elements = [];
				//Find all model elements 
				ModelElements.find({}, function (err, modelElements) {
					if (err) {
						ModelElements.closeConnection();
						callback(err);
					}
					var wbsJoin = wbs.join();
					//TO DO: Investigate optimization of this process
					//Filter out model elements that are not present on any of the whiteboard being exported
					//References among elements currently not explicitely handled
					var i = modelElements.length
					while (i--) {
						//get a model element's references
						var refs=modelElements[i].get("references");
						if(!refs){//element is not instantiated on any whiteboard
							rejected_elements[modelElements[i]._id]=modelElements[i];
							modelElements.splice(i, 1);//we remove it
							continue;
						}						
						refs = Object.keys(refs);//convert refs keys to array
						var keep = false;
						for(var j=0; j<refs.length;j++){
							if(wbsJoin.includes(refs[j])){
								var ref_model_el_id=modelElements[i].get("cell").ref_model_id;
								if(ref_model_el_id){//we will retrieve these model elements later
									ref_model_ids_to_retrieve.push(ref_model_el_id);
								}
								keep= true;
								break;	
							}

						}
						if (!keep) {
							rejected_elements[modelElements[i]._id]=modelElements[i];
							modelElements.splice(i, 1);
						} 
					}
					//retrieve inter model element posponed references 
					for(var i=0; i<ref_model_ids_to_retrieve.length; i++){
						if(rejected_elements[ref_model_ids_to_retrieve[i]]){
							modelElements.push( rejected_elements[ref_model_ids_to_retrieve[i]])
							rejected_elements[ref_model_ids_to_retrieve[i]]=null; //ensure model element is not added multiple times
						}

					}

					workManager.workStarted();
					var meFile = expImpPath + '/' + 'modelElements.json';
					fs.outputJson(meFile, modelElements, { spaces: 4 }, function (err) {
						ModelElements.closeConnection();
						if (err) callback(err);
						winston.info('Exported Model Elements to - ', meFile);
						workManager.workEnded();
					});
					workManager.workEnded();
					callback(null);
				});
			},
			function(callback){
				workManager.workStarted();
				var DesignRoom = require('../models/designroom');
				var p_id =  config['projectDB'];
				//Find all model elements 
				DesignRoom.findOne({_id:p_id}, function (err, liveProject) {
					if (err) {
						workManager.workEnded()
						callback(err);
					}
					var designRoomFile = expImpPath + '/' + 'designroom.json';
					liveProject = liveProject.toObject();

					//remove non needed properties for whiteboard exports
					delete liveProject['adminGroup'];
					delete liveProject['normalGroup'];
					delete liveProject['readOnlyGroup'];

					fs.outputJson(designRoomFile, liveProject, { spaces: 4 }, function (err) {
						if (err) callback(err);
						winston.info('Exported live project to - ', designRoomFile);
						callback(null);
						workManager.workEnded();

					});
				});
			}
		],
			function (err, result) {
				// all done
				if (Rooms != null) {
					Rooms.closeConnection();
				}

				workManager.workEnded();
		});

	};

	return Exporter;
};