/************************************************************************/
/* Licensed Materials – Property of  HCL                                */
/* (c) Copyright HCL Technologies Ltd. 2018 All rights reserved.  */
/* Trademark of HCL Technologies Limited                                */
/************************************************************************/

	var workManager = require('../modules/workManager');
	var request = require('request');
	var Requester = {};
	var port=config.sslPort || 443;
	var host=process.env.VCAP_APP_HOST || "localhost";
	var livePort = config['port'] || port;
	var baseURL = "https://"+host+":"+livePort;
	var authServer = config["authServerURL"];
	var homeServerURL = config['homeServerURL']+':'+port;
	
	Requester.authenticate = function (username, password, done) {
		//set options
		var reqOptions = { method: 'POST',
		url: authServer+'/loginLocal',
		headers: 
		 { 'content-type': 'application/x-www-form-urlencoded',
		   'cache-control': 'no-cache' 
		 },
		 agentOptions: {
			rejectUnauthorized: false
		  },
		form: { username: username, password: password } };
		
		//send request
	  request(reqOptions, function (error, response, body) {
			//err
			if (error) {
				winston.error("There seems to be a server issue");
				process.exit(1);
			}

			//proceed
			winston.info("Token is generated");
			Requester.setToken(JSON.parse(body).token);
			done(body);
	  });
	
	}
	Requester.setToken = function(token){
		Requester["token"] = token;
		winston.info("Token is set")
	}
	Requester.getToken = function(token){
		return Requester["token"]?Requester["token"]:null;
	}
	Requester.addCells = function (room_id, graph_id, cells, done ){
		//set options
		var reqOptions = { method: 'POST',
		url: baseURL+'/api/addCells',
		headers: 
		 { 'content-type': 'application/x-www-form-urlencoded',
			 'cache-control': 'no-cache',
			 'x-access-token': Requester.getToken()
		 },
		 agentOptions: {
			rejectUnauthorized: false
		  },
			form: { room_id: room_id, graph_id: graph_id, cells : cells } 
		};
		
		//place request
	  request(reqOptions, function (error, response, body) {
			if (error) return done(error);

			//send response
			winston.info("whiteboard:"+graph_id+ " in "+room_id+" was cleared successfully");
			done(null);
	  });

	}

	Requester.clearWhiteboard = function (room_id, graph_id, done ){
		//set options
		var reqOptions = { method: 'DELETE',
		url: baseURL+'/api/clearWhiteboard',
		headers: 
		 { 'content-type': 'application/x-www-form-urlencoded',
			 'cache-control': 'no-cache',
			 'x-access-token': Requester.getToken(),
			 'authorization': 'Bearer '+Requester.getToken()//support for new mps architecture
		 },
		 agentOptions: {
			rejectUnauthorized: false
		  },
			form: { room_id: room_id, graph_id: graph_id } 
		};
		
		//place request
	  request(reqOptions, function (error, response, body) {
			if (error) return done(error);

			//send response
			winston.info("whiteboard:"+graph_id+ " in "+room_id+" was cleared successfully");
			done(null);
	  });

	}


	Requester.getAllLocalGroups = function (done){
		//set options
		var reqOptions = { method: 'GET',
		url: authServer+'/groups',
		headers: 
		 { 'content-type': 'application/x-www-form-urlencoded',
			 'cache-control': 'no-cache',
			 'x-access-token': Requester.getToken(),
			 'authorization': 'Bearer '+Requester.getToken()//support for new mps architecture
		 },
		 agentOptions: {
			rejectUnauthorized: false
		  }//,
	//		form: { room_id: room_id, graph_id: graph_id } 
		};
		
		//place request
	  request(reqOptions, function (error, response, body) {
			if (error) return done(error);

			var localGroups = JSON.parse(body);
			winston.info(localGroups);
			for(var i =localGroups.length-1; i>=0;i--){
				if(localGroups[i].ad){
					localGroups.splice(i,1);
				}
			}
			//send response
			done(null, localGroups);
	  });

	}

	Requester.getAllUsers = function (done){
		//set options
		var reqOptions = { method: 'GET',
		url: authServer+'/users',
		headers: 
		 { 'content-type': 'application/x-www-form-urlencoded',
			 'cache-control': 'no-cache',
			 'x-access-token': Requester.getToken(),
			 'authorization': 'Bearer '+Requester.getToken()//support for new mps architecture
		 },
		 qs:{
			 'includePassword':true
			},
		 agentOptions: {
			rejectUnauthorized: false
		  }//,
	//		form: { room_id: room_id, graph_id: graph_id } 
		};
		
		//place request
	  request(reqOptions, function (error, response, body) {
			if (error) return done(error);

			//send response
			done(null, body);
	  });

	}
	Requester.getAllRooms = function (done){
		//set options
		var reqOptions = { method: 'GET',
		url: homeServerURL+'/designrooms',
		headers: 
		 { 'content-type': 'application/x-www-form-urlencoded',
			 'cache-control': 'no-cache',
			 'authorization': 'Bearer '+Requester.getToken()//support for new mps architecture
		 },
		 agentOptions: {
			rejectUnauthorized: false
		  }
		};
		
		//place request
	  request(reqOptions, function (error, response, body) {
			if (error) return done(error);

			if(body && body.length){
				body = JSON.parse(body);
			}
			//send response
			done(null, body);
	  });

	}

	Requester.resolveAllModelChanges = function (done){
		//set options
		var reqOptions = { method: 'PUT',
		url: baseURL+'/modelElements/resolve',
		headers: 
		 { 'content-type': 'application/x-www-form-urlencoded',
			 'cache-control': 'no-cache',
			 'x-access-token': Requester.getToken(),
			 'authorization': 'Bearer '+Requester.getToken()//support for new mps architecture
		 },
		 agentOptions: {
			rejectUnauthorized: false
		  }
		};
		
		//place request
	  request(reqOptions, function (error, response, body) {
			if (error) return done(error);

			//send response
			winston.info("Model changes propagations completed successfully");
			done(null);
	  });

	}
	module.exports = Requester;
