/************************************************************************/
/* Licensed Materials – Property of  HCL                                */
/* (c) Copyright HCL Technologies Ltd. 2016, 2017 All rights reserved.  */
/* Trademark of HCL Technologies Limited                                */
/************************************************************************/

const fs = require('fs-extra');
const async = require('async');
var workManager = require('../modules/workManager');
var Groups = require('../models/group');
var Requester = require('./requester');;
var groupsFileIds = null;
var adminAccessMap = {};

module.exports = function () {
	var Importer = {};

	Importer.doImport = function (expImpPath, includeADUsers) {
		var config = require('../modules/config');
		var Users = require('../models/user');
		var groupsFile = expImpPath + '/groups.json'
		var usersFile = expImpPath + '/users.json';

		async.waterfall([
			function (callback) {
				workManager.workStarted();
                winston.info('Importing groups');
				fs.readJson(groupsFile, { spaces: 4 }, function (err, groups) {
					if(err){
						winston.error(err);
					}// do not stop if impossible to import groups
					callback(null, groups);
				});
			},

			function generateAdminAccessMap(groups, callback){
				if(groups){//no groups to import
					//Home admin so all All rooms present
					requester.getAllRooms(function(err, rooms){
						if(err){
							winson.error(err);
							callback(err);
						}
						if(rooms && rooms.length){
							for(var i=0; i<rooms.length; i++){
								//If admin group for room valid
								if(rooms[i].adminGroup && rooms[i].adminGroup.id){
									//Add adminG dependency to map
									if(!adminAccessMap[rooms[i].adminGroup.id]){//group id entry not in map yet
										adminAccessMap[rooms[i].adminGroup.id]= {adminGroup:{}};
										adminAccessMap[rooms[i].adminGroup.id]['adminGroup'][rooms[i].adminGroup.id] =[rooms[i].adminGroup.id];
									}
									else{//group id entry already in map
										adminAccessMap[rooms[i].adminGroup.id]['adminGroup'][rooms[i].adminGroup.id].push(rooms[i].adminGroup.id);
									}

									//Normal group
									if(rooms[i].normalGroup && rooms[i].normalGroup.id){
										//Add adminG dependency to map
										if(!adminAccessMap[rooms[i].normalGroup.id]){//group id entry not in map yet
											adminAccessMap[rooms[i].normalGroup.id]= {adminGroup:{}};
											adminAccessMap[rooms[i].normalGroup.id]['adminGroup'][rooms[i].adminGroup.id] =[rooms[i].adminGroup.id];
										}
										else{//group id entry already in map
											adminAccessMap[rooms[i].normalGroup.id]['adminGroup'][rooms[i].adminGroup.id].push(rooms[i].adminGroup.id);
										}
									}

									//read only group
									if(rooms[i].readOnlyGroup && rooms[i].readOnlyGroup.id){
										//Add adminG dependency to map
										if(!adminAccessMap[rooms[i].readOnlyGroup.id]){//group id entry not in map yet
											adminAccessMap[rooms[i].readOnlyGroup.id]= {adminGroup:{}};
											adminAccessMap[rooms[i].readOnlyGroup.id]['adminGroup'][rooms[i].adminGroup.id] =[rooms[i].adminGroup.id];
										}
										else{//group id entry already in map
											adminAccessMap[rooms[i].readOnlyGroup.id]['adminGroup'][rooms[i].adminGroup.id].push(rooms[i].adminGroup.id);
										}
									}

								}
							}
							groups.forEach(function(group){//update incoming file groups admin list dependencies
								if(adminAccessMap[group.id]){
									group['adminGroup'] = adminAccessMap[group.id]['adminGroup'];
								}
								else{
									group['adminGroup'] = null;
								}

							});

							callback(null, groups);

						}else{
							callback(null, groups);
						}

					});
				}
				else{
					callback(null, groups);
				}
			},
			function(groups, callback){
				if(groups){// import groups
					groupsFileIds = Array.from(groups, function(x){ return x.id; });
					var groupsFileNames = Array.from(groups, function(x){ return x.groupname; });
					//USE AUTH SERVER API?
					Groups.deleteMany({$or:[{id: {$in :groupsFileIds}}, {groupname:{$in: groupsFileNames}}]}, function(err, res){
						if(err){
							winston.error("Issue occured when groups from db");
							callback(err);
						}
						else{
							Groups.insertMany(groups, function (err) {
								if(!err) winston.info('Groups inserted:', groups);
								callback(err);
							});							
						}
					});				
				}
				else{//no group to import. Proceed
					winston.info('No groups to import');
					callback();
				}

			},			
			// read users from file
			function (callback) {
                winston.info('Importing users');
				fs.readJson(usersFile, { spaces: 4 }, function (err, users) {
					callback(err, users);
				});
			},
			// remove records currently present in db
			function (users, callback) {
				Users.find({}, function (err, usersDB) {
					var adminExists= false;//admin already present in db
					var adminsToRemove = [];
					var adFileUsernames = [];
					var i = usersDB.length-1;
					while(i>=0){
						if(usersDB[i]._doc.local.username=="admin"){
							if(adminExists==false){
								adminExists=true;
							}
							else{
								adminsToRemove.push(usersDB[i]._id);
							}
						}
						i--;
					}
					var usersDBIds = Array.from(usersDB, function(x){ return x._id; });
					for(var i = 0; i<usersDBIds.length; i++){

						var j=users.length-1;
						while(j>=0){
							if(!includeADUsers && users[j].ad) {//if option set, filter out AD users
								adFileUsernames.push(users[j].local.username)
								users.splice(j,1);
								j--;
								continue;
							}

							if( usersDBIds[i]==users[j]._id || (adminExists==true && users[j].local.username=="admin")){ //filter out new admins as admin already exists
								users.splice(j,1);
							}
							j--;
						}
					}
					//if duplicate admins in db, remove
					if(adminsToRemove.length){
						winston.info("Removing " +adminsToRemove.length+" duplicate admin from db");
						Users.deleteMany({_id: {$in :adminsToRemove}}, function(err, res){
							if(err){
								winston.error("Issue occured when deleting duplicate admin from db");
								callback(err);
							}
							else{
								callback(err, users, adFileUsernames);
							}
						});
					}else{
						callback(err, users, adFileUsernames);
					}
					
				});
			},
			// insert the users read from file
			function (users, adFileUsernames ,callback) {
				// now insert
				Users.insertMany(users, function (err) {
					var usersInserted = Array.from(users, function(x){ return x.id; });
					callback(err, adFileUsernames);
				});
			},
			function(adFileUsernames, callback){
				if(!includeADUsers && groupsFileIds){ //only if ad users are NOT included AND there **exists groups to update
					//delete all AD users from all 
					var asyncTasks = [];

					// remove all ad users from all **currently being imported groups 
					adFileUsernames.forEach(function (username) {
						asyncTasks.push(asyncDeleteUserFromGroups(username));
					});
					//winston.info(asyncTasks);
					async.parallel(asyncTasks,
						// optional callback
						function(err, results) {
							if(err){
								callback(err);
							}
							//log result 
							winston.info("Async Parallel Group Update done", results);
							callback(err);
						});				
				}
				else{
					callback();
				}
			}
		],
			function (err, result) {
				// all done
                if (err) {
					winston.info('Import Groups & Users failed - Error - ', err);
				}

				if (Users != null) {
					Users.closeConnection();
				}

				workManager.workEnded();
			});

	};
	//Delete user from all groups
	function asyncDeleteUserFromGroups(username){

		var func = function(callback){
			Groups.update(
				{id:{$in :groupsFileIds}},
				{ $pull: { users: username } },
				{ multi: true },
				function(err, numberAffected) {
					if(err) callback(err);
					winston.info('Number of rows affected', numberAffected);
					callback(err);
				});		
			}
		return func;
	}
	return Importer;
};