/************************************************************************/
/* Licensed Materials – Property of  HCL                                */
/* (c) Copyright HCL Technologies Ltd. 2018 All rights reserved.  */
/* Trademark of HCL Technologies Limited                                */
/************************************************************************/

const fs = require('fs-extra');
var archiver = require("archiver");
var workManager = require('../modules/workManager');

var Zipper={};

//Ensures zips folder is present
Zipper.createZipFolder = function(zipFolderPath, done) {
	winston.info('Creating zip data folder', zipFolderPath);
	fs.ensureDir(zipFolderPath, function(err){
		done(err);
	});
}
Zipper.doZip = function (sourcePath, designationPath, done) {
	//var zipFileOutputPath = Zipper.zip_file_path + '.zip';
	winston.info('Writing zip Source from: ' + sourcePath);
	winston.info('Writing zip Destination file to:'+ designationPath);

	// create a file to stream archive data to.
	var output = fs.createWriteStream(designationPath);
	var archive = archiver('zip', {
		zlib: { level: 0 } //Zero compression level, just store, does not fire Zlib 
	});

	// listen for all archive data to be written
	// 'close' event is fired only when a file descriptor is involved
	output.on('close', function () {
		winston.info(archive.pointer() + ' total bytes');
		winston.info('archiver has been finalized and the output file descriptor has closed.');
		done(null);
	});

	// This event is fired when the data source is drained no matter what was the data source.
	// It is not part of this library but rather from the NodeJS Stream API.
	// @see: https://nodejs.org/api/stream.html#stream_event_end
	output.on('end', function () {
		winston.info('Data has been drained');
	});

	// good practice to catch warnings (ie stat failures and other non-blocking errors)
	archive.on('warning', function (err) {
		if (err.code === 'ENOENT') {
			// log warning
		} else {
			// throw error
			winston.error(err);
			done(err);
		}
	});

	// good practice to catch this error explicitly
	archive.on('error', function (err) {
		winston.error(err);
		done(err);
	});

	// pipe archive data to the file
	archive.pipe(output);

	// append files from a sub-directory and naming it `new-subdir` within the archive
	archive.directory(sourcePath, sourcePath.substring(sourcePath.lastIndexOf('/')));

	// finalize the archive (ie we are done appending files but streams have to finish yet)
	// 'close', 'end' or 'finish' may be fired right after calling this method so register to them beforehand
	archive.finalize();


}

module.exports = Zipper;